"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Methods = require("./methods");
/**
 * AsyncArray Class
 *
 * @class AsyncArray
 * @extends {Array<T>}
 * @template T
 */
class AsyncArray extends Array {
    /**
     * Creates an instance of AsyncArray
     * @param {...T[]} args
     * @memberof AsyncArray
     */
    constructor(...args) {
        super(...args);
    }
    /**
     * Async Every method
     *
     * @param {Methods.CallBackFilter<T>} cb
     * @returns {Promise<boolean>}
     * @memberof AsyncArray
     */
    aEvery(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.every(this, cb);
        });
    }
    /**
     * Async Filter method
     *
     * @param {Methods.CallBackFilter<T>} cb
     * @returns {Promise<T[]>}
     * @memberof AsyncArray
     */
    aFilter(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return new AsyncArray(...(yield Methods.filter(this, cb)));
        });
    }
    /**
     * Async find method
     *
     * @param {Methods.CallBackFind<T>} cb
     * @returns {Promise<T | undefined>}
     * @memberof AsyncArray
     */
    aFind(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.find(this, cb);
        });
    }
    /**
     * Async findIndex method
     *
     * @param {Methods.CallBackFindIndex<T>} cb
     * @returns {Promise<number>}
     * @memberof AsyncArray
     */
    aFindIndex(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.findIndex(this, cb);
        });
    }
    /**
     * Async ForEach method
     *
     * @param {Methods.CallBackForEach<T>} cb
     * @returns {Promise<void>}
     * @memberof AsyncArray
     */
    aForEach(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Methods.forEach(this, cb);
        });
    }
    /**
     * Async Map method
     *
     * @template R
     * @param {Methods.CallBackMap<T, R>} cb
     * @returns {Promise<R[]>}
     * @memberof AsyncArray
     */
    aMap(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return new AsyncArray(...(yield Methods.map(this, cb)));
        });
    }
    /**
     * Async Reduce method
     *
     * @template R
     * @param {Methods.CallBackReduce<T, R>} cb
     * @param {R} [initialValue]
     * @returns {Promise<T | R>}
     * @memberof AsyncArray
     */
    aReduce(cb, initialValue) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.reduce(this, cb, initialValue);
        });
    }
    /**
     * Async ReduceRight method
     *
     * @template R
     * @param {Methods.CallBackReduceRight<T, R>} cb
     * @param {R} [initialValue]
     * @returns {Promise<T | R>}
     * @memberof AsyncArray
     */
    aReduceRight(cb, initialValue) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.reduceRight(this, cb, initialValue);
        });
    }
    /**
     * Async Some method
     *
     * @param {Methods.CallBackFilter<T>} cb
     * @returns {Promise<boolean>}
     * @memberof AsyncArray
     */
    aSome(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.some(this, cb);
        });
    }
}
exports.AsyncArray = AsyncArray;
//# sourceMappingURL=async_ray.js.map